@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests of status command.
 
@@ -656,7 +656,7 @@ class TestStatusEncodings(TestCaseWithTransport):
         self.stdout = sys.stdout
 
     def tearDown(self):
-        bzrlib.user_encoding = self.user_encoding
+        osutils._cached_user_encoding = self.user_encoding
         sys.stdout = self.stdout
         TestCaseWithTransport.tearDown(self)
 
