@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 """Tests being able to ignore bad filetypes."""
@@ -25,7 +25,7 @@ from bzrlib import (
     errors,
     )
 from bzrlib.status import show_tree_status
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import OsFifoFeature, TestCaseWithTransport
 
 
 def verify_status(tester, tree, value):
@@ -40,10 +40,7 @@ class TestBadFiles(TestCaseWithTransport):
 
     def test_bad_files(self):
         """Test that bzr will ignore files it doesn't like"""
-        if getattr(os, 'mkfifo', None) is None:
-            # TODO: Ultimately this should be TestSkipped
-            # or PlatformDeficiency
-            return
+        self.requireFeature(OsFifoFeature)
 
         wt = self.make_branch_and_tree('.')
         b = wt.branch
