@@ -23,8 +23,11 @@ import re
 import sys
 import time
 
-import bzrlib
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    tests,
+    ui as _mod_ui,
+    )
 from bzrlib.symbol_versioning import (
     deprecated_in,
     )
@@ -33,36 +36,30 @@ from bzrlib.tests import (
     TestUIFactory,
     StringIOWrapper,
     )
-from bzrlib.tests.test_progress import _TTYStringIO
+from bzrlib.tests.test_progress import (
+    _NonTTYStringIO,
+    _TTYStringIO,
+    )
 from bzrlib.ui import (
+    CannedInputUIFactory,
     CLIUIFactory,
     SilentUIFactory,
+    UIFactory,
+    make_ui_for_terminal,
     )
 from bzrlib.ui.text import (
+    NullProgressView,
     TextProgressView,
     TextUIFactory,
     )
 
 
-class UITests(TestCase):
-
-    def test_silent_factory(self):
-        ui = SilentUIFactory()
-        stdout = StringIO()
-        self.assertEqual(None,
-                         self.apply_redirected(None, stdout, stdout,
-                                               ui.get_password))
-        self.assertEqual('', stdout.getvalue())
-        self.assertEqual(None,
-                         self.apply_redirected(None, stdout, stdout,
-                                               ui.get_password,
-                                               u'Hello\u1234 %(user)s',
-                                               user=u'some\u1234'))
-        self.assertEqual('', stdout.getvalue())
+class UITests(tests.TestCase):
 
     def test_text_factory_ascii_password(self):
-        ui = TestUIFactory(stdin='secret\n', stdout=StringIOWrapper(),
-                           stderr=StringIOWrapper())
+        ui = tests.TestUIFactory(stdin='secret\n',
+                                 stdout=tests.StringIOWrapper(),
+                                 stderr=tests.StringIOWrapper())
         pb = ui.nested_progress_bar()
         try:
             self.assertEqual('secret',
@@ -83,9 +80,9 @@ class UITests(TestCase):
         We can't predict what encoding users will have for stdin, so we force
         it to utf8 to test that we transport the password correctly.
         """
-        ui = TestUIFactory(stdin=u'baz\u1234'.encode('utf8'),
-                           stdout=StringIOWrapper(),
-                           stderr=StringIOWrapper())
+        ui = tests.TestUIFactory(stdin=u'baz\u1234'.encode('utf8'),
+                                 stdout=tests.StringIOWrapper(),
+                                 stderr=tests.StringIOWrapper())
         ui.stderr.encoding = ui.stdout.encoding = ui.stdin.encoding = 'utf8'
         pb = ui.nested_progress_bar()
         try:
@@ -103,6 +100,56 @@ class UITests(TestCase):
         finally:
             pb.finished()
 
+    def test_progress_construction(self):
+        """TextUIFactory constructs the right progress view.
+        """
+        for (file_class, term, pb, expected_pb_class) in (
+            # on an xterm, either use them or not as the user requests,
+            # otherwise default on
+            (_TTYStringIO, 'xterm', 'none', NullProgressView),
+            (_TTYStringIO, 'xterm', 'text', TextProgressView),
+            (_TTYStringIO, 'xterm', None, TextProgressView),
+            # on a dumb terminal, again if there's explicit configuration do
+            # it, otherwise default off
+            (_TTYStringIO, 'dumb', 'none', NullProgressView),
+            (_TTYStringIO, 'dumb', 'text', TextProgressView),
+            (_TTYStringIO, 'dumb', None, NullProgressView),
+            # on a non-tty terminal, it's null regardless of $TERM
+            (StringIO, 'xterm', None, NullProgressView),
+            (StringIO, 'dumb', None, NullProgressView),
+            # however, it can still be forced on
+            (StringIO, 'dumb', 'text', TextProgressView),
+            ):
+            os.environ['TERM'] = term
+            if pb is None:
+                if 'BZR_PROGRESS_BAR' in os.environ:
+                    del os.environ['BZR_PROGRESS_BAR']
+            else:
+                os.environ['BZR_PROGRESS_BAR'] = pb
+            stdin = file_class('')
+            stderr = file_class()
+            stdout = file_class()
+            uif = make_ui_for_terminal(stdin, stdout, stderr)
+            self.assertIsInstance(uif, TextUIFactory,
+                "TERM=%s BZR_PROGRESS_BAR=%s uif=%r" % (term, pb, uif,))
+            self.assertIsInstance(uif.make_progress_view(),
+                expected_pb_class,
+                "TERM=%s BZR_PROGRESS_BAR=%s uif=%r" % (term, pb, uif,))
+
+    def test_text_ui_non_terminal(self):
+        """Even on non-ttys, make_ui_for_terminal gives a text ui."""
+        stdin = _NonTTYStringIO('')
+        stderr = _NonTTYStringIO()
+        stdout = _NonTTYStringIO()
+        for term_type in ['dumb', None, 'xterm']:
+            if term_type is None:
+                del os.environ['TERM']
+            else:
+                os.environ['TERM'] = term_type
+            uif = make_ui_for_terminal(stdin, stdout, stderr)
+            self.assertIsInstance(uif, TextUIFactory,
+                'TERM=%r' % (term_type,))
+
     def test_progress_note(self):
         stderr = StringIO()
         stdout = StringIO()
@@ -122,14 +169,15 @@ class UITests(TestCase):
             pb.finished()
 
     def test_progress_note_clears(self):
-        stderr = StringIO()
-        stdout = StringIO()
-        # The PQM redirects the output to a file, so it
-        # defaults to creating a Dots progress bar. we
-        # need to force it to believe we are a TTY
+        stderr = _TTYStringIO()
+        stdout = _TTYStringIO()
+        # so that we get a TextProgressBar
+        os.environ['TERM'] = 'xterm'
         ui_factory = TextUIFactory(
             stdin=StringIO(''),
             stdout=stdout, stderr=stderr)
+        self.assertIsInstance(ui_factory._progress_view,
+            TextProgressView)
         pb = ui_factory.nested_progress_bar()
         try:
             # Create a progress update that isn't throttled
@@ -158,43 +206,30 @@ class UITests(TestCase):
         pb2.finished()
         pb1.finished()
 
-    def assert_get_bool_acceptance_of_user_input(self, factory):
-        factory.stdin = StringIO("y\nyes with garbage\n"
-                                 "yes\nn\nnot an answer\n"
-                                 "no\nfoo\n")
-        factory.stdout = StringIO()
-        factory.stderr = StringIO()
-        # there is no output from the base factory
-        self.assertEqual(True, factory.get_boolean(""))
+    def test_text_ui_get_boolean(self):
+        stdin = StringIO("y\n" # True
+                         "n\n" # False
+                         "yes with garbage\nY\n" # True
+                         "not an answer\nno\n" # False
+                         "I'm sure!\nyes\n" # True
+                         "NO\n" # False
+                         "foo\n")
+        stdout = StringIO()
+        stderr = StringIO()
+        factory = TextUIFactory(stdin, stdout, stderr)
         self.assertEqual(True, factory.get_boolean(""))
         self.assertEqual(False, factory.get_boolean(""))
+        self.assertEqual(True, factory.get_boolean(""))
+        self.assertEqual(False, factory.get_boolean(""))
+        self.assertEqual(True, factory.get_boolean(""))
         self.assertEqual(False, factory.get_boolean(""))
         self.assertEqual("foo\n", factory.stdin.read())
         # stdin should be empty
         self.assertEqual('', factory.stdin.readline())
 
-    def test_silent_ui_getbool(self):
-        factory = SilentUIFactory()
-        self.assert_get_bool_acceptance_of_user_input(factory)
-
-    def test_silent_factory_prompts_silently(self):
-        factory = SilentUIFactory()
-        stdout = StringIO()
-        factory.stdin = StringIO("y\n")
-        self.assertEqual(True,
-                         self.apply_redirected(None, stdout, stdout,
-                                               factory.get_boolean, "foo"))
-        self.assertEqual("", stdout.getvalue())
-        # stdin should be empty
-        self.assertEqual('', factory.stdin.readline())
-
-    def test_text_ui_getbool(self):
-        factory = TextUIFactory(None, None, None)
-        self.assert_get_bool_acceptance_of_user_input(factory)
-
     def test_text_factory_prompt(self):
         # see <https://launchpad.net/bugs/365891>
-        factory = TextUIFactory(None, StringIO(), StringIO(), StringIO())
+        factory = TextUIFactory(StringIO(), StringIO(), StringIO())
         factory.prompt('foo %2e')
         self.assertEqual('', factory.stdout.getvalue())
         self.assertEqual('foo %2e', factory.stderr.getvalue())
@@ -202,6 +237,7 @@ class UITests(TestCase):
     def test_text_factory_prompts_and_clears(self):
         # a get_boolean call should clear the pb before prompting
         out = _TTYStringIO()
+        os.environ['TERM'] = 'xterm'
         factory = TextUIFactory(stdin=StringIO("yada\ny\n"), stdout=out, stderr=out)
         pb = factory.nested_progress_bar()
         pb.show_bar = False
@@ -232,17 +268,6 @@ class UITests(TestCase):
         finally:
             pb.finished()
 
-    def test_silent_ui_getusername(self):
-        factory = SilentUIFactory()
-        factory.stdin = StringIO("someuser\n\n")
-        factory.stdout = StringIO()
-        factory.stderr = StringIO()
-        self.assertEquals(None,
-            factory.get_username(u'Hello\u1234 %(host)s', host=u'some\u1234'))
-        self.assertEquals("", factory.stdout.getvalue())
-        self.assertEquals("", factory.stderr.getvalue())
-        self.assertEquals("someuser\n\n", factory.stdin.getvalue())
-
     def test_text_ui_getusername(self):
         factory = TextUIFactory(None, None, None)
         factory.stdin = StringIO("someuser\n\n")
@@ -259,8 +284,9 @@ class UITests(TestCase):
         self.assertEqual('', factory.stdin.readline())
 
     def test_text_ui_getusername_utf8(self):
-        ui = TestUIFactory(stdin=u'someuser\u1234'.encode('utf8'),
-                           stdout=StringIOWrapper(), stderr=StringIOWrapper())
+        ui = tests.TestUIFactory(stdin=u'someuser\u1234'.encode('utf8'),
+                                 stdout=tests.StringIOWrapper(),
+                                 stderr=tests.StringIOWrapper())
         ui.stderr.encoding = ui.stdout.encoding = ui.stdin.encoding = "utf8"
         pb = ui.nested_progress_bar()
         try:
@@ -275,61 +301,112 @@ class UITests(TestCase):
             pb.finished()
 
 
-class TestTextProgressView(TestCase):
-    """Tests for text display of progress bars.
-    """
-    # XXX: These might be a bit easier to write if the rendering and
-    # state-maintaining parts of TextProgressView were more separate, and if
-    # the progress task called back directly to its own view not to the ui
-    # factory. -- mbp 20090312
+class CLIUITests(TestCase):
+
+    def test_cli_factory_deprecated(self):
+        uif = self.applyDeprecated(deprecated_in((1, 18, 0)),
+            CLIUIFactory,
+            StringIO(), StringIO(), StringIO())
+        self.assertIsInstance(uif, UIFactory)
+
+
+class SilentUITests(TestCase):
+
+    def test_silent_factory_get_password(self):
+        # A silent factory that can't do user interaction can't get a
+        # password.  Possibly it should raise a more specific error but it
+        # can't succeed.
+        ui = SilentUIFactory()
+        stdout = StringIO()
+        self.assertRaises(
+            NotImplementedError,
+            self.apply_redirected,
+            None, stdout, stdout, ui.get_password)
+        # and it didn't write anything out either
+        self.assertEqual('', stdout.getvalue())
+
+    def test_silent_ui_getbool(self):
+        factory = SilentUIFactory()
+        stdout = StringIO()
+        self.assertRaises(
+            NotImplementedError,
+            self.apply_redirected,
+            None, stdout, stdout, factory.get_boolean, "foo")
+
+
+class TestUIFactoryTests(TestCase):
+
+    def test_test_ui_factory_progress(self):
+        # there's no output; we just want to make sure this doesn't crash -
+        # see https://bugs.edge.launchpad.net/bzr/+bug/408201
+        ui = TestUIFactory()
+        pb = ui.nested_progress_bar()
+        pb.update('hello')
+        pb.tick()
+        pb.finished()
+
+
+class CannedInputUIFactoryTests(TestCase):
     
-    def _make_factory(self):
-        out = StringIO()
-        uif = TextUIFactory(stderr=out)
-        uif._progress_view._width = 80
-        return out, uif
+    def test_canned_input_get_input(self):
+        uif = CannedInputUIFactory([True, 'mbp', 'password'])
+        self.assertEqual(uif.get_boolean('Extra cheese?'), True)
+        self.assertEqual(uif.get_username('Enter your user name'), 'mbp')
+        self.assertEqual(uif.get_password('Password for %(host)s', host='example.com'),
+            'password')
 
-    def test_render_progress_easy(self):
-        """Just one task and one quarter done"""
-        out, uif = self._make_factory()
-        task = uif.nested_progress_bar()
-        task.update('reticulating splines', 5, 20)
-        self.assertEqual(
-'\r[####/               ] reticulating splines 5/20                               \r'
-            , out.getvalue())
 
-    def test_render_progress_nested(self):
-        """Tasks proportionally contribute to overall progress"""
-        out, uif = self._make_factory()
-        task = uif.nested_progress_bar()
-        task.update('reticulating splines', 0, 2)
-        task2 = uif.nested_progress_bar()
-        task2.update('stage2', 1, 2)
-        # so we're in the first half of the main task, and half way through
-        # that
-        self.assertEqual(
-r'[####\               ] reticulating splines:stage2 1/2'
-            , uif._progress_view._render_line())
-        # if the nested task is complete, then we're all the way through the
-        # first half of the overall work
-        task2.update('stage2', 2, 2)
-        self.assertEqual(
-r'[#########|          ] reticulating splines:stage2 2/2'
-            , uif._progress_view._render_line())
+class TestBoolFromString(tests.TestCase):
 
-    def test_render_progress_sub_nested(self):
-        """Intermediate tasks don't mess up calculation."""
-        out, uif = self._make_factory()
-        task_a = uif.nested_progress_bar()
-        task_a.update('a', 0, 2)
-        task_b = uif.nested_progress_bar()
-        task_b.update('b')
-        task_c = uif.nested_progress_bar()
-        task_c.update('c', 1, 2)
-        # the top-level task is in its first half; the middle one has no
-        # progress indication, just a label; and the bottom one is half done,
-        # so the overall fraction is 1/4
-        self.assertEqual(
-            r'[####|               ] a:b:c 1/2'
-            , uif._progress_view._render_line())
+    def assertIsTrue(self, s, accepted_values=None):
+        res = _mod_ui.bool_from_string(s, accepted_values=accepted_values)
+        self.assertEquals(True, res)
 
+    def assertIsFalse(self, s, accepted_values=None):
+        res = _mod_ui.bool_from_string(s, accepted_values=accepted_values)
+        self.assertEquals(False, res)
+
+    def assertIsNone(self, s, accepted_values=None):
+        res = _mod_ui.bool_from_string(s, accepted_values=accepted_values)
+        self.assertIs(None, res)
+
+    def test_know_valid_values(self):
+        self.assertIsTrue('true')
+        self.assertIsFalse('false')
+        self.assertIsTrue('1')
+        self.assertIsFalse('0')
+        self.assertIsTrue('on')
+        self.assertIsFalse('off')
+        self.assertIsTrue('yes')
+        self.assertIsFalse('no')
+        self.assertIsTrue('y')
+        self.assertIsFalse('n')
+        # Also try some case variations
+        self.assertIsTrue('True')
+        self.assertIsFalse('False')
+        self.assertIsTrue('On')
+        self.assertIsFalse('Off')
+        self.assertIsTrue('ON')
+        self.assertIsFalse('OFF')
+        self.assertIsTrue('oN')
+        self.assertIsFalse('oFf')
+
+    def test_invalid_values(self):
+        self.assertIsNone(None)
+        self.assertIsNone('doubt')
+        self.assertIsNone('frue')
+        self.assertIsNone('talse')
+        self.assertIsNone('42')
+
+    def test_provided_values(self):
+        av = dict(y=True, n=False, yes=True, no=False)
+        self.assertIsTrue('y', av)
+        self.assertIsTrue('Y', av)
+        self.assertIsTrue('Yes', av)
+        self.assertIsFalse('n', av)
+        self.assertIsFalse('N', av)
+        self.assertIsFalse('No', av)
+        self.assertIsNone('1', av)
+        self.assertIsNone('0', av)
+        self.assertIsNone('on', av)
+        self.assertIsNone('off', av)
